/*-------------------------------------------------------------------
[SCRIPT] Demo optimiseur et statistiques.sql
[DATABASE] Optimizer_Demos
[DESCRIPTION] Dmonstration estimation de cardinalits
[MAJ PAR] DATAFLY - Arian Papillon
[DATEMAJ] 2014 
-------------------------------------------------------------------*/
USE StatsDemo
GO
--------------------------------------------------------------------------
-- DBCC SHOW_STATISTICS
--------------------------------------------------------------------------
DBCC SHOW_STATISTICS (Clients, nix$client$nom)

-- Densit = 1.576183E-06 -> 634444 lignes distinctes = 1 / Densit)
SELECT 1 / 1.576183E-06
SELECT COUNT(DISTINCT Nom) FROM Clients

--------------------------------------------------------------------------
-- Recherche d'galit avec une constante : utilisation de l'histogramme 
--------------------------------------------------------------------------
SELECT Nom from Clients WHERE Nom = N'Adam Khan'
   -- valeur de l'histogramme : 7 lignes
SELECT Nom from Clients WHERE Nom = N'Adrienne Rush' 
	-- ?

-- Parameter sniffing ?  
SELECT DB_ID()
DBCC FLUSHPROCINDB (9)  -- DBCC FREEPROCCACHE  


--------------------------------------------------------------------------
-- Recherche d'galit avec une variable : utilisation de la densit
--------------------------------------------------------------------------
DECLARE @VNom NVARCHAR(50) = N'Adam Khan'
SELECT Nom from Clients WHERE Nom = @VNom  
    -- 1.60577 -> NbLignes * Densit    
    -- SELECT 1018774 * 1.576183E-06
GO
-- OPTIMIZE FOR UNKNOWN
DECLARE @VNom NVARCHAR(50) = N'Adam Khan'
SELECT Nom from Clients WHERE Nom = @VNom OPTION (OPTIMIZE FOR UNKNOWN)
GO
-- OPTIMIZE FOR valeur
DECLARE @VNom NVARCHAR(50) = N'Adam Khan'
SELECT Nom from Clients WHERE Nom = @VNom OPTION (OPTIMIZE FOR (@VNom = 'Adam Khan'))  -- 7 lignes
GO

-- SQL Dynamique
DECLARE @VNom NVARCHAR(50) = N'Adam Khan'
DECLARE @sql nvarchar(max) = 'SELECT Nom from Clients WHERE Nom = '''+@VNom+''''
EXEC (@sql)
GO


--------------------------------------------------------------------------
-- Recherche d'ingalit
--------------------------------------------------------------------------
-- Avec valeur connue, se sert de l'histogramme
SELECT Nom from Clients WHERE Nom < N'Adam Khan'  
-- SELECT Nom from Clients WHERE Pays <> N'Germany'  
-- Plage
SELECT Nom from Clients WHERE Nom  BETWEEN N'Adam Khan' and N'Adrienne Rush'  -- valeur de l'histogramme + range_rows : 3556 lignes = 6+3543+7
-- Distinct
-- SELECT DISTINCT Nom from Clients WHERE Nom  BETWEEN N'Adam Khan' and N'Adrienne Rush'  -- Nb de valeurs de l'histogramme + distinct_range_rows : 2135 lignes = 2133+1+1


-- Sans valeur connue, la densit ne sert  rien
DECLARE @VNom NVARCHAR(50) = N'Adam Khan', @VNom2 NVARCHAR(50) = N'Adrienne Rush'  
-- Ingalit avec une borne
SELECT Nom from Clients WHERE Nom < @VNom	  				                       
    -- 305632 lignes    SELECT 1018774 * 30/100  -- (30 %)
-- Ingalit avec deux bornes, SQL 2012
SELECT Nom from Clients WHERE Nom BETWEEN @VNom AND @VNom2  OPTION (QUERYTRACEON 9481)  -- 91689.7
    -- 91689.7 lignes   SELECT 1018774 * 9/100  -- (9 %)
-- Deux bornes, SQL 2014
SELECT Nom from Clients WHERE Nom BETWEEN @VNom AND @VNom2  OPTION (QUERYTRACEON 2312)  -- 167402 
    -- Nouvelle formule : ???


--------------------------------------------------------------------------
-- PAS DE STATISTIQUES 
--------------------------------------------------------------------------
-- Egalit
-- Estimateur SQL Server 2012
SELECT Nom from ClientsNoStats WHERE Nom = N'Adam Khan' OPTION (QUERYTRACEON 9481) 
    -- 32067 lignes pour 7 !     SELECT SQRT(1018774 * SQRT(1018774))  -- (ou 1018774 Exposant 0.75)
GO
-- Estimateur SQL Server 2014
SELECT Nom from ClientsNoStats WHERE Nom = N'Adam Khan' OPTION (QUERYTRACEON 2312) 
    -- 1009.34 lignes pour 7 !   SELECT sqrt(1018774)
GO
-- Ingalit 
SELECT Nom from ClientsNoStats WHERE Nom < N'Adam Khan'  
    -- 305632 lignes pour 4789     SELECT 1018774 * 30/100  -- (30 %)
--SELECT Nom from ClientsNoStats WHERE Nom BETWEEN N'Adam Khan' AND N'Adrienne Rush' -- 9%

GO


--------------------------------------------------------------------------
-- PREDICATS MULTIPLES 
--------------------------------------------------------------------------
SELECT IdClient FROM Clients WHERE Region = 'Seine (Paris)'  -- 37007 lignes
SELECT IdClient FROM Clients WHERE Ville = 'Paris'           -- 37004 lignes  

-- Valeurs connues : utilisation de l'histogramme
-------------------------------------------------
-- Estimation version 2012
SELECT IdClient FROM Clients WHERE Ville = 'Paris' AND Region = 'Seine (Paris)' OPTION (QUERYTRACEON 9481) 
    -- 1344.17 lignes pour 37004
    -- Formule : (L_Paris x L_Seine)/L_Total
    -- SELECT 37007 * 37004 / 1018774.0

-- Estimation version 2014
SELECT IdClient FROM Clients WHERE Ville = 'Paris' AND Region = 'Seine (Paris)' OPTION (QUERYTRACEON 2312)  
    -- 7052.64 lignes pour 37004
    -- Formule : L_Seine x SQRT(L_Paris)  / SQRT(L_Total)
    -- SELECT 37004 * SQRT(37007) / SQRT(1018774.0)

---- Forcer le cacul avec les traceflags------------
---- forcer la slectivit minimum 2008/2012 - fix 2008R2SP1CU5-2012CU1 et traceflag 4137
SELECT IdClient FROM Clients WHERE Ville = 'Paris' AND Region = 'Seine (Paris)' OPTION (QUERYTRACEON 9481, QUERYTRACEON 4137) 
---- forcer la slectivit minimum 2014 - traceflags 9471 (et 9472)
SELECT IdClient FROM Clients WHERE Ville = 'Paris' AND Region = 'Seine (Paris)' OPTION (QUERYTRACEON 2312, QUERYTRACEON 9471)  

-- Forcer le calcul avec des statistiques supplmentaires
-- Et si nous crions des statistiques multi-colonnes ?
CREATE STATISTICS ClientsVillesRegions ON Clients (Ville,Region) WITH FULLSCAN
DBCC FLUSHPROCINDB (9)   
-- 34934,7  => SELECT 37007 * (0.003289474 / 0.003484321)   -- Lignes_region_Seine * (densit_villeregion / densit_ville)

-- Et si nous crions des statistiques filtres ?
CREATE STATISTICS ClientsVille_RegionSeine ON Clients (Ville) WHERE Region = 'Seine (Paris)' WITH FULLSCAN
DBCC FLUSHPROCINDB (9) 


-- DROP STATISTICS
DROP STATISTICS Clients.ClientsVillesRegions
DROP STATISTICS Clients.ClientsVille_RegionSeine

-- Valeurs inconnues : utilisation de la densit
------------------------------------------------
DECLARE @VVille NVARCHAR(30) = 'Paris', @VRegion NVARCHAR(50) = 'Seine (Paris)'
-- Estimation version 2012
SELECT IdClient FROM Clients WHERE Ville = @VVille AND Region = @VRegion OPTION (QUERYTRACEON 9481) 
    -- 63.3881 lignes pour 37004
    -- formule : D_Ville x D_Region x L_Total
    -- SELECT 0.003484321 * 0.01785714 * 1018774
-- Estimation version 2014
SELECT IdClient FROM Clients WHERE Ville = @VVille AND Region = @VRegion OPTION (QUERYTRACEON 2312) 
    -- 474.353 lignes pour 37004
    -- Formule : D_Ville x SQRT(D_Region) x L_Total
    -- SELECT 0.003484321 * SQRT(0.01785714) * 1018774

-- Et si nous crions des statistiques multi-colonnes ?
CREATE STATISTICS ClientsVillesRegions ON Clients (Ville,Region) WITH FULLSCAN
DBCC FLUSHPROCINDB (9)   
-- Densit multi-colonne SELECT 0.003289474 * 1018774
-- ... mais sont-elles utilises par les deux versions...?

--------------------------------------------------------------------------
-- JOINTURE
--------------------------------------------------------------------------
SELECT NomProduit FROM Produits p JOIN Fournisseurs f 
ON p.IdFournisseur = f.IdFournisseur
WHERE f.Nom = 'Microsoft'
-- 15.92 lignes estimes pour 14 lignes
-- Produit     : Nombre de lignes = 5000, Densit IdFournisseur = 0.001592357    
-- Fournisseur : Microsoft -> histogramme = 2 lignes
-- Formule     : (NbLignesTotal_Produit x Densit_Produit_IdFournisseur) x Nb_Lignes_pour_Microsoft  
   SELECT (5000 * 0.001592357) * 2  

--------------------------------------------------------------------------
-- LIKE : String Summary Statistics
--------------------------------------------------------------------------
-- dans les 40 premiers caractres
SELECT DescProduit FROM produits WHERE DescProduit LIKE '%ABCDE%'
-- dans les 40 derniers caractres
SELECT DescProduit FROM produits WHERE DescProduit LIKE '%FGHIJ%'
-- au milieu (aprs les 40 premiers et avant les 40 derniers caractres)
SELECT DescProduit FROM produits WHERE DescProduit LIKE '%KLMN%'


