/*-------------------------------------------------------------------
[SCRIPT] Statistics_report.sql
[DATABASE] StatsDemo
[DESCRIPTION] Rapport des statistiques
[MAJ PAR] DATAFLY - Arian Papillon
[DATEMAJ]  Novembre 2014
-------------------------------------------------------------------*/
USE StatsDemo
GO

-- Using sys.dm_db_stats_properties
----------------------------------
SELECT  SCHEMA_NAME(o.schema_id) AS schema_name
      , OBJECT_NAME(s.object_id) AS object_name
      , s.name AS stats_name
      , sp.last_updated
      , sc.stats_column_id
      , c.name AS column_name
      , tc.table_rows
      , sp.rows AS stat_rows
      , CAST (( CAST(sp.rows_sampled AS DECIMAL(18, 3)) / ( sp.rows ) ) * 100 AS DECIMAL(18,
                                                              2)) AS sample
      , s.filter_definition
      , sp.modification_counter
FROM    sys.stats AS s
        JOIN sys.objects o ON s.object_id = o.object_id
        LEFT JOIN ( SELECT   t.object_id
                      , SUM(p.rows) AS table_rows
               FROM     sys.objects t
                        JOIN sys.indexes i ON t.object_id = i.object_id
                        JOIN sys.partitions p ON i.object_id = p.object_id
                                                 AND i.index_id = p.index_id
               WHERE    t.type IN ('U','V')
                        AND i.index_id < 2
               GROUP BY t.object_id
             ) AS tc ON o.object_id = tc.object_id
        INNER JOIN sys.stats_columns AS sc ON s.stats_id = sc.stats_id
                                              AND s.object_id = sc.object_id
--                                              AND sc.stats_column_id = 1  -- premire colonne seulement
        INNER JOIN sys.columns AS c ON c.object_id = s.object_id
                                       AND c.object_id = sc.object_id
                                       AND c.column_id = sc.column_id
        CROSS APPLY sys.dm_db_stats_properties(s.object_id, s.stats_id) sp
WHERE   OBJECTPROPERTY(s.object_id, 'IsUserTable') = 1
        AND OBJECTPROPERTY(s.object_id, 'IsMSShipped') = 0
        AND OBJECT_NAME(s.object_id) <> 'sysdiagrams' -- seulement tables utilisateur
ORDER BY OBJECT_NAME(s.object_id)
      , s.name
      , sc.stats_column_id
